!WRF:DRIVER_LAYER:INTEGRATION
!

MODULE module_integrate
!chacha
   real, dimension(:,:,:), allocatable :: u_ra, u_rb
   real, dimension(:,:,:), allocatable :: v_ra, v_rb

   real, dimension(30) :: sig_a              !half level eta value
!chacha


CONTAINS

RECURSIVE SUBROUTINE integrate ( grid )



   USE module_domain
   USE module_driver_constants
   USE module_nesting
   USE module_configure
   USE module_timing
   USE module_utility

   IMPLICIT NONE

   !  Input data.

   TYPE(domain) , POINTER :: grid

!chacha
   TYPE(WRFU_TimeInterval) :: sp_bdy_increment
   TYPE(WRFU_Time)         :: next_time, current_time
   CHARACTER*40            :: current_datestr,next_datestr
   CHARACTER*19            :: current_date
!chacha

! module_integrate:integrate
! <DESCRIPTION> 
! This is a driver-level routine that controls the integration of a
! domain and subdomains rooted at the domain. 
! 
! The integrate routine takes a domain pointed to by the argument
! <em>grid</em> and advances the domain and its associated nests from the
! grid's current time, stored within grid%domain_clock, to a given time
! forward in the simulation, stored as grid%stop_subtime. The
! stop_subtime value is arbitrary and does not have to be the same as
! time that the domain finished integrating.  The simulation stop time
! for the grid is known to the grid's clock (grid%domain_clock) and that
! is checked with a call to domain_clockisstoptime prior to beginning the
! loop over time period that is specified by the
! current time/stop_subtime interval.
! 
! The clock, the simulation stop time for the domain, and other timing
! aspects for the grid are set up in the routine
! (<a href="setup_timekeeping.html">setup_timekeeping</a>) at the time
! that the domain is initialized.
! The lower-level time library and the type declarations for the times
! and time intervals used are defined either in 
! external/esmf_time_f90/module_utility.F90 or in 
! external/io_esmf/module_utility.F90 depending on a build-time decision to 
! incorporate either the embedded ESMF subset implementation contained in 
! external/esmf_time_f90 or to use a site-specific installation of the ESMF 
! library.  This decision is made during the configuration step of the WRF 
! build process.  Note that arithmetic and comparison is performed on these 
! data types using F90 operator overloading, also defined in that library.
! 
! This routine is the lowest level of the WRF Driver Layer and for the most
! part the WRF routines that are called from here are in the topmost level
! of the Mediation Layer.  Mediation layer routines typically are not 
! defined in modules. Therefore, the routines that this routine calls
! have explicit interfaces specified in an interface block in this routine.
!
! As part of the Driver Layer, this routine is intended to be non model-specific
! and so a minimum of WRF-specific logic is coded at this level. Rather, there
! are a number of calls to mediation layer routines that contain this logic, some
! of which are merely stubs in WRF Mediation Layer that sits below this routine
! in the call tree.  The routines that integrate calls in WRF are defined in
! share/mediation_integrate.F.
! 
! Flow of control
! 
! 1. Check to see that the domain is not finished 
! by testing the value returned by domain_clockisstoptime for the
! domain.
! 
! 2. <a href=model_to_grid_config_rec.html>Model_to_grid_config_rec</a> is called to load the local config_flags
! structure with the configuration information for the grid stored
! in model_config_rec and indexed by the grid's unique integer id. These
! structures are defined in frame/module_configure.F.
! 
! 3. The current time of the domain is retrieved from the domain's clock
! using domain_get_current_time.  
! 
! 4. Iterate forward while the current time is less than the stop subtime.
! 
! 4.a. Start timing for this iteration (only on node zero in distributed-memory runs)
! 
! 4.b. Call <a href=med_setup_step.html>med_setup_step</a> to allow the mediation layer to 
! do anything that's needed to call the solver for this domain.  In WRF this means setting
! the indices into the 4D tracer arrays for the domain.
! 
! 4.c. Check for any nests that need to be started up at this time.  This is done 
! calling the logical function <a href=nests_to_open.html>nests_to_open</a> (defined in 
! frame/module_nesting.F) which returns true and the index into the current domain's list
! of children to use for the nest when one needs to be started.
! 
! 4.c.1  Call <a href=alloc_and_configure_domain.html>alloc_and_configure_domain</a> to allocate
! the new nest and link it as a child of this grid.
! 
! 4.c.2  Call <a href=setup_Timekeeping.html>setup_Timekeeping</a> for the nest.
! 
! 4.c.3  Initialize the nest's arrays by calling <a href=med_nest_initial.html>med_nest_initial</a>. This will
! either interpolate data from this grid onto the nest, read it in from a file, or both. In a restart run, this
! is also where the nest reads in its restart file.
! 
! 4.d  If a nest was opened above, check for and resolve overlaps (this is not implemented in WRF 2.0, which
! supports multiple nests on the same level but does not support overlapping).
! 
! 4.e  Give the mediation layer an opportunity to do something before the solver is called by
! calling <a href=med_before_solve_io.html>med_before_solve_io</a>. In WRF this is the point at which history and
! restart data is output.
! 
! 4.f  Call <a href=solve_interface.html>solve_interface</a> which calls the solver that advance the domain 
! one time step, then advance the domain's clock by calling domain_clockadvance.  
! The enclosing WHILE loop around this section is for handling other domains 
! with which this domain may overlap.  It is not active in WRF 2.0 and only 
! executes one trip.  
! 
! 4.g Call med_calc_model_time and med_after_solve_io, which are stubs in WRF.
! 
! 4.h Iterate over the children of this domain (<tt>DO kid = 1, max_nests</tt>) and check each child pointer to see
! if it is associated (and therefore, active).
! 
! 4.h.1  Force the nested domain boundaries from this domain by calling <a href=med_nest_force.html>med_nest_force</a>.
! 
! 4.h.2  Setup the time period over which the nest is to run. Sine the current grid has been advanced one time step
! and the nest has not, the start for the nest is this grid's current time minus one time step.  The nest's stop_subtime
! is the current time, bringing the nest up the same time level as this grid, its parent.
! 
! 4.h.3  Recursively call this routine, integrate, to advance the nest's time.  Since it is recursive, this will
! also advance all the domains who are nests of this nest and so on.  In other words, when this call returns, all
! the domains rooted at the nest will be at the current time.
! 
! 4.h.4  Feedback data from the nested domain back onto this domain by calling <a href=med_nest_feedback.html>med_nest_feedback</a>.
! 
! 4.i  Write the time to compute this grid and its subtree. This marks the end of the loop begun at step 4, above.
! 
! 5. Give the mediation layer an opportunity to do I/O at the end of the sequence of steps that brought the
! grid up to stop_subtime with a call to <a href=med_last_solve_io.html>med_last_solve_io</a>.  In WRF, this 
! is used to generate the final history and/or restart output when the domain reaches the end of it's integration.
! There is logic here to make sure this occurs correctly on a nest, since the nest may finish before its parent.
! </DESCRIPTION>

   !  Local data.

   CHARACTER*32                           :: outname, rstname
   TYPE(domain) , POINTER                 :: grid_ptr , new_nest
   TYPE(domain)                           :: intermediate_grid
   INTEGER                                :: step
   INTEGER                                :: nestid , kid
   LOGICAL                                :: a_nest_was_opened
   INTEGER                                :: fid , rid
   LOGICAL                                :: lbc_opened
   REAL                                   :: time, btime, bfrq
   CHARACTER*256                          :: message, message2
   TYPE (grid_config_rec_type)            :: config_flags
   LOGICAL , EXTERNAL                     :: wrf_dm_on_monitor
   INTEGER                                :: idum1 , idum2 , ierr , open_status
   LOGICAL                                :: should_do_last_io
!chacha
   INTEGER                                :: rc
   real :: xtimea, xtimeb
   integer :: stime    ! xtime * 60 = second
   integer :: ids,ide,jds,jde,kds,kde
   integer :: ii,jj,kk
   character(len=80) :: filename
!chacha


   ! interface
   INTERFACE
       ! mediation-supplied solver
     SUBROUTINE solve_interface ( grid )
       USE module_domain
       TYPE (domain) grid
     END SUBROUTINE solve_interface
       ! mediation-supplied routine to allow driver to pass time information
       ! down to mediation/model layer
     SUBROUTINE med_calc_model_time ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_calc_model_time
       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! perform I/O before the call to the solve routine
     SUBROUTINE med_before_solve_io ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_before_solve_io
       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! perform I/O after the call to the solve routine
     SUBROUTINE med_after_solve_io ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_after_solve_io
       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! perform I/O to initialize a new nest
     SUBROUTINE med_pre_nest_initial ( parent , newid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain), POINTER ::  parent
       INTEGER, INTENT(IN)    ::  newid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_pre_nest_initial
     SUBROUTINE med_nest_initial ( parent , grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain), POINTER ::  grid , parent
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_nest_initial
       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! provide parent->nest forcing
     SUBROUTINE med_nest_force ( parent , grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain), POINTER ::  grid, parent
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_nest_force

#ifdef MOVE_NESTS
     SUBROUTINE med_nest_move ( parent , grid )
       USE module_domain
       USE module_configure
       TYPE (domain), POINTER ::  grid , parent
     END SUBROUTINE med_nest_move
#endif

       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! provide parent->nest feedback
     SUBROUTINE med_nest_feedback ( parent , grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain), POINTER ::  grid , parent
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_nest_feedback

       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! perform I/O prior to the close of this call to integrate
     SUBROUTINE med_last_solve_io ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_last_solve_io
       ! mediation-supplied routine that gives mediation layer opportunity to 
       ! perform setup before iteration over steps in this call to integrate
     SUBROUTINE med_setup_step ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_setup_step
       ! mediation-supplied routine that gives mediation layer opportunity to
       ! perform setup before iteration over steps in this call to integrate
     SUBROUTINE med_endup_step ( grid , config_flags )
       USE module_domain
       USE module_configure
       TYPE (domain) grid
       TYPE (grid_config_rec_type) config_flags
     END SUBROUTINE med_endup_step
       ! mediation-supplied routine that intializes the nest from the grid
       ! by interpolation

     SUBROUTINE Setup_Timekeeping( grid )
       USE module_domain
       TYPE(domain), POINTER :: grid
     END SUBROUTINE

   END INTERFACE

!chacha
   grid_ptr => grid

   ids=grid_ptr%sm31+3
   ide=grid_ptr%em31-3
   kds=grid_ptr%sm32
   kde=grid_ptr%em32
   jds=grid_ptr%sm33+3
   jde=grid_ptr%em33-3

   allocate (u_ra(ide-ids+1,jde-jds,kde-kds))
   allocate (u_rb(ide-ids+1,jde-jds,kde-kds))

   allocate (v_ra(ide-ids,jde-jds+1,kde-kds))
   allocate (v_rb(ide-ids,jde-jds+1,kde-kds))

!   allocate (t_ra(ide-ids,jde-jds,kde-kds))
!   allocate (t_rb(ide-ids,jde-jds,kde-kds))

   current_time=domain_get_current_time(grid)
   CALL model_to_grid_config_rec ( grid%id , model_config_rec , config_flags )
   CALL WRFU_ClockGet( grid%domain_clock, CurrTime=current_time, rc=rc )
   CALL wrf_timetoa( domain_get_current_time(grid), current_datestr )!chacha
   !print*,current_time, current_time%basetime%S

   filename='wrfinput'//current_datestr
   print*,"filename=",filename
!chacha


   ! This allows us to reference the current grid from anywhere beneath 
   ! this point for debugging purposes.  
   ! In the case of operations involving already initialized parent and child, 
   ! the current grid is set to the child grid.  
!$$$ Improve this later to track both parent and child...  
!$$$ Use either an optional argument or another interface...  
   CALL set_current_grid_ptr( grid )

   call read_wrf_nc(filename,u_rb, v_rb)

   xtimeb=grid%xtime
   xtimea=grid%xtime+3600.

   sig_a(:)=grid%em_znu(:)

   call WRFU_TimeIntervalSet( sp_bdy_increment, H=6,rc=rc)
   next_time = domain_get_current_time(grid) + sp_bdy_increment
   CALL wrf_timetoa( next_time, next_datestr )
   filename='wrfinput'//next_datestr
   print*,"next filename=",filename
   call read_wrf_nc(filename, u_ra, v_ra)


   IF ( .NOT. domain_clockisstoptime( grid ) ) THEN
      CALL model_to_grid_config_rec ( grid%id , model_config_rec , config_flags )
      IF ( .NOT. grid%return_after_training_io ) THEN
         CALL domain_clockprint ( 150, grid, 'DEBUG:  top of integrate(),' )
      ENDIF
      DO WHILE ( .NOT. domain_clockisstopsubtime(grid) )
         IF ( .NOT. grid%return_after_training_io ) THEN
            IF ( wrf_dm_on_monitor() ) THEN
              CALL start_timing
            END IF
         ENDIF
         CALL med_setup_step ( grid , config_flags )
         IF ( .NOT. grid%return_after_training_io ) THEN
            a_nest_was_opened = .false.
            ! for each nest whose time has come...
            DO WHILE ( nests_to_open( grid , nestid , kid ) )
               ! nestid is index into model_config_rec (module_configure) of the grid
               ! to be opened; kid is index into an open slot in grid's list of children
               a_nest_was_opened = .true.
               CALL med_pre_nest_initial ( grid , nestid , config_flags )
               CALL alloc_and_configure_domain ( domain_id  = nestid ,   &
                                                 grid       = new_nest , &
                                                 parent     = grid ,     &
                                                 kid        = kid        )
               CALL Setup_Timekeeping (new_nest)
               CALL med_nest_initial ( grid , new_nest , config_flags )
            END DO
            IF ( a_nest_was_opened ) THEN
               CALL set_overlaps ( grid )   ! find overlapping and set pointers
            END IF
         ENDIF
         CALL med_before_solve_io ( grid , config_flags )
         IF ( grid%return_after_training_io ) THEN
           CALL wrf_debug( 1, 'DEBUG:  module_integrate() returned after training' )
           RETURN  ! an ugly hack for sure, only needed for ESMF
         ENDIF
         grid_ptr => grid
         DO WHILE ( ASSOCIATED( grid_ptr ) )
            CALL set_current_grid_ptr( grid_ptr )
            CALL wrf_debug( 100 , 'module_integrate: calling solve interface ' )
            CALL solve_interface ( grid_ptr ) 
            CALL domain_clockadvance ( grid_ptr )
!chacha
           current_time=domain_get_current_time(grid)

           if(mod(grid%xtime,3600).eq.0)then  !xtime's unit is minute
!date

             CALL wrf_timetoa( current_time, current_datestr )
             filename='wrfinput'//current_datestr
             print*,"read lowbdy file from ",filename
             !call read_wrf_lowbdy(filename,                            &
             !                grid_ptr%sst(ids:ide,jds:jde),       &
             !                grid_ptr%vegfra(ids:ide,jds:jde),    &
             !                grid_ptr%albedo(ids:ide,jds:jde) )

             CALL WRFU_TimeIntervalSet( sp_bdy_increment, H=6,rc=rc)
             next_time = domain_get_current_time(grid) + sp_bdy_increment
             CALL wrf_timetoa( next_time, next_datestr )
             filename='wrfinput'//next_datestr
             print*,"next file name=",filename
!date

             u_rb(:,:,:)=u_ra(:,:,:)
             v_rb(:,:,:)=v_ra(:,:,:)

             call read_wrf_nc(filename, u_ra, v_ra)
!             print*,"sst=",grid%sst(50,50)
!             print*,"vegfra=",grid%vegfra(50,50)
!             print*,"albedo=",grid%albedo(50,50)

            xtimeb=grid%xtime
            xtimea=grid%xtime+3600.
            !stime=int(grid%xtime*60.)

           endif

         print*,"test!!!"
         print*,"u_rb=",u_rb(50,50,10),"u_ra=",u_ra(50,50,10)
         print*,"v_rb=",v_rb(50,50,10),"v_ra=",v_ra(50,50,10)
         print*,"xtime=",grid%xtime,"xtimea=",xtimea,"xtimeb=",xtimeb

!         if(mod(stime,450) == 0)then
!         print*,'stime=',stime
!         !spectral nudging u
         call spectral(grid_ptr%em_u_2(ids:ide,kds:kde-1,jds:jde-1),     &
                       u_ra,u_rb,grid%xtime,xtimea,xtimeb,ids,ide,jds,jde-1,kds,kde-1)
!         !spectral nudging v
         call spectral(grid_ptr%em_v_2(ids:ide-1,kds:kde-1,jds:jde),     &
                       v_ra,v_rb,grid%xtime,xtimea,xtimeb,ids,ide-1,jds,jde,kds,kde-1)
!         endif

!chacha

            CALL wrf_debug( 100 , 'module_integrate: back from solve interface ' )
            ! print lots of time-related information for testing
            ! switch this on with namelist variable self_test_domain
            CALL domain_time_test( grid_ptr, 'domain_clockadvance' )
            grid_ptr => grid_ptr%sibling
         END DO
         CALL set_current_grid_ptr( grid )
         CALL med_calc_model_time ( grid , config_flags )
         CALL med_after_solve_io ( grid , config_flags )
         grid_ptr => grid
         DO WHILE ( ASSOCIATED( grid_ptr ) )
            DO kid = 1, max_nests
              IF ( ASSOCIATED( grid_ptr%nests(kid)%ptr ) ) THEN
                CALL set_current_grid_ptr( grid_ptr%nests(kid)%ptr )
                ! Recursive -- advance nests from previous time level to this time level.
                CALL wrf_debug( 100 , 'module_integrate: calling med_nest_force ' )
                CALL med_nest_force ( grid_ptr , grid_ptr%nests(kid)%ptr , config_flags )
                CALL wrf_debug( 100 , 'module_integrate: back from med_nest_force ' )
                grid_ptr%nests(kid)%ptr%start_subtime = &
                  domain_get_current_time(grid) - domain_get_time_step(grid)
                grid_ptr%nests(kid)%ptr%stop_subtime = &
                  domain_get_current_time(grid)
                CALL integrate ( grid_ptr%nests(kid)%ptr ) 
                CALL wrf_debug( 100 , 'module_integrate: back from recursive call to integrate ' )
                CALL wrf_debug( 100 , 'module_integrate: calling med_nest_feedback ' )
                CALL med_nest_feedback ( grid_ptr , grid_ptr%nests(kid)%ptr , config_flags )
                CALL wrf_debug( 100 , 'module_integrate: back from med_nest_feedback ' )
#ifdef MOVE_NESTS
                IF ( .NOT. domain_clockisstoptime( head_grid ) ) THEN
                  CALL med_nest_move ( grid_ptr , grid_ptr%nests(kid)%ptr )
                ENDIF
#endif
              END IF
            END DO
            grid_ptr => grid_ptr%sibling
         END DO
         CALL set_current_grid_ptr( grid )
         !  Report on the timing for a single time step.
         IF ( wrf_dm_on_monitor() ) THEN
! begin KLUDGE
! ia32 pgi 6.0-2 and prescribed moving nest: the returned date string message2 is
! corrupt after the first specified move UNLESS the string is initialized to all 
! spaces - REMOVE THIS INIT OF MESSAGE2 WHEN POSSIBLE 
! 2005 10 28
            message2='                                                                ' // &
                     '                                                                ' // &
                     '                                                                ' // &
                     '                                                                '
! end KLUDGE
           CALL domain_clock_get ( grid, current_timestr=message2 )
           WRITE ( message , FMT = '("main: time ",A," on domain ",I3)' ) TRIM(message2), grid%id
           CALL end_timing ( TRIM(message) )
         END IF
         CALL med_endup_step ( grid , config_flags )
      END DO
      ! Avoid double writes on nests if this is not really the last time;
      ! Do check for write if the parent domain is ending.
      IF ( grid%id .EQ. 1 ) THEN               ! head_grid
        CALL med_last_solve_io ( grid , config_flags )
      ELSE

! zip up the tree and see if any ancestor is at its stop time
        
        should_do_last_io = domain_clockisstoptime( head_grid )
        grid_ptr => grid 
        DO WHILE ( grid_ptr%id .NE. 1 )
           IF ( domain_clockisstoptime( grid_ptr ) ) should_do_last_io = .TRUE. 
           grid_ptr => grid_ptr%parents(1)%ptr
        ENDDO
        IF ( should_do_last_io ) THEN 
           CALL med_last_solve_io ( grid , config_flags )
        ENDIF
      ENDIF
   END IF

END SUBROUTINE integrate

SUBROUTINE spectral(fval, avala, avalb, ztime,ztimea, ztimeb,  ids, ide, jds, jde, kds, kde)
!     zvfk : intensification factor for vertical nesting
!     fval : forecast variables at next time step
!     aval : large-scale variables at the correspponding time
!     avala: 6 hourly interval large-scale variables at later   time step
!     avalb: 6 hourly interval large-scale variables at earlier time step
!     index: variable index to identify cross or dot point value

      real, parameter :: zvfk = 0.1
      real      :: ztime, ztimea, ztimeb
      integer   :: ids,ide,jds,jde,kds,kde

      real, dimension(ide-ids+1,kde-kds+1,jde-jds+1) :: fval
      real, dimension(ide-ids+1,jde-jds+1,kde-kds+1) :: aval
      real, dimension(ide-ids+1,jde-jds+1,kde-kds+1) :: avala, avalb

!     fspec, aspec: internal storage fields for spectral nudging

      real, dimension(ide-ids+1,jde-jds+1) :: fspec,   aspec

!     spectral coefficients for in- and output fields

      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aa_s, ab_s, ba_s, bb_s
      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aar_s, abr_s, bar_s, bbr_s
      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aa_n, ab_n, ba_n, bb_n

!     dimensions for local fields
!     zwgtwd       : vertical weighting profile
!     ftmp1, ftmp2 : temporary local array for forecast value
!     atmp1, atmp2 : temporary local array for analysis value

      real,dimension(kde-kds+1) :: zwgtvd
      real,dimension(ide-ids+1,jde-jds+1) :: ftmp1, atmp1
      real,dimension(ide-ids+1,jde-jds+1) :: ftmp2, atmp2

!c.... preparations
!chacha
!$omp parallel do &
!$omp private(ii,jj)
!chacha
      do 10 ii = 1, ((ide-ids)/2+1)
      do 10 jj = 1, ((jde-jds)/2+1)
         aa_n(ii,jj) = 0.
         ab_n(ii,jj) = 0.
         ba_n(ii,jj) = 0.
         bb_n(ii,jj) = 0.
   10 continue
!$omp end parallel do


!.... Linear interpolation to determine large-scale values at corresponding time

      xm = ztime  - ztimeb
      xn = ztimea - ztime
      !print*,"xm=",xm,"xn=",xn
!chacha
!$omp parallel do &
!$omp private(kk,ii,jj)
      do 20 kk = 1,kde-kds+1
      do 20 jj = 1,jde-jds+1
      do 20 ii = 1,ide-ids+1
      aval(ii,jj,kk) = (xm*avala(ii,jj,kk)+xn*avalb(ii,jj,kk))/(xm+xn)
   20 continue
!$omp end parallel do
!.... assign of required local fields

!      do 30 k = 1,kde-kds+1
!         if (a(k).lt.0.85) k85 = k
!   30 continue 
      do 31 k = 1,kde-kds+1
         zwgtvd(k) = 0.5*zvfk*(1.0-min(sig_a(k),1.0)/1.0)**2
!         zwgtvd(k) = 0.5*0.1
   31 continue

!.... preparation of spectral decomposition: calculation of (is,js) from (i,j)

      isc = 2
      jsc = 2

      is  = (ide-ids)/2 + 1
      js  = (jde-jds)/2 + 1
!chacha
!!cc$omp parallel do default(shared)
!!cc$omp private(k)
!chacha
      do 100 k = 1,kde-kds+1
!chacha
!$omp parallel do &
!$omp private(ii,jj)
!chacha
      do 40 jj = 1,jde-jds+1
      do 40 ii = 1,ide-ids+1
         fspec(ii,jj) = fval(ii,k,jj)
         aspec(ii,jj) = aval(ii,jj,k)
   40 continue
!$omp end parallel do
      call speczer(ide-ids+1,jde-jds+1,is,js,isc,jsc,fspec,aa_s ,ab_s, ba_s ,bb_s)
      call speczer(ide-ids+1,jde-jds+1,is,js,isc,jsc,aspec,aar_s,abr_s,bar_s,bbr_s)

!chacha
!$omp parallel do &
!$omp private(ii,jj)
!chacha
      do 70 jj = 1,jsc
      do 70 ii = 1,isc 
         aa_n(ii,jj) = aa_s(ii,jj) - zwgtvd(k)*(aa_s(ii,jj)-aar_s(ii,jj))
         ab_n(ii,jj) = ab_s(ii,jj) - zwgtvd(k)*(ab_s(ii,jj)-abr_s(ii,jj))
         ba_n(ii,jj) = ba_s(ii,jj) - zwgtvd(k)*(ba_s(ii,jj)-bar_s(ii,jj))
         bb_n(ii,jj) = bb_s(ii,jj) - zwgtvd(k)*(bb_s(ii,jj)-bbr_s(ii,jj))
   70 continue
!$omp end parallel do

!c.... re-composition of spectral coefficients to fields
      call speczus(ide-ids+1,jde-jds+1,is,js,isc,jsc,                   &
           fspec,aa_s,ab_s,ba_s,bb_s,aa_n,ab_n,ba_n,bb_n)

!chacha
!$omp parallel do &
!$omp private(ii,jj)
!chacha
      do 90 jj = 1,jde-jds+1
      do 90 ii = 1,ide-ids+1
        fval(ii,k,jj) = fspec(ii,jj)
   90 continue
!$omp end parallel do
  
  100 continue

      return
      end subroutine spectral

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine speczer                                          &
                (ii,jj,is,js,isc,jsc,feld,aafin,abfin,bafin,bbfin)

      dimension feld(ii,jj)  
      dimension asem(is,jj)  , bsem(is,jj)
      dimension aafin(is,js), abfin(is,js)
      dimension bafin(is,js), bbfin(is,js)


      twopi = 8.*atan(1.)
!chacha
!$omp parallel do &
!$omp private(ms,ns)
!chacha
      do 10 ns = 1,js
      do 10 ms = 1,is
         aafin(ms,ns) = 0.
         abfin(ms,ns) = 0.
         bafin(ms,ns) = 0.
         bbfin(ms,ns) = 0.
   10 continue
!$omp end parallel do

!chacha
!$omp parallel do &
!$omp private(ms,n)
!chacha
      do 20 n  = 1,jj
      do 20 ms = 1,is
         asem(ms,n) = 0.
                                                      bsem(ms,n) = 0.
   20 continue
!$omp end parallel do

!.... In x-direction

!chacha
!$omp parallel do &
!$omp private(ms,n)
!chacha
      do 30 n  = 1,jj
      do 30 ms = 1,isc
      do 30 ir = 1,ii
         asem(ms,n) = asem(ms,n)+(feld(ir,n)*cos(twopi*real(ms-1)*      &
                     real(ir)/real(ii)))/real(ii)
         bsem(ms,n) = bsem(ms,n)+(feld(ir,n)*sin(twopi*real(ms-1)*      &
                     real(ir)/real(ii)))/real(ii)
   30 continue

!.... In Y-direction

!chacha
!$omp parallel do &
!$omp private(ms,ns)
!chacha
      do 40 ms = 1,isc
      do 40 ns = 1,jsc
      do 40 jr = 1,jj
         aafin(ms,ns) = aafin(ms,ns)+(asem(ms,jr)*cos(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         bafin(ms,ns) = bafin(ms,ns)+(asem(ms,jr)*sin(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         abfin(ms,ns) = abfin(ms,ns)+(bsem(ms,jr)*cos(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         bbfin(ms,ns) = bbfin(ms,ns)+(bsem(ms,jr)*sin(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
   40 continue
      return
      end subroutine speczer
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1

      subroutine speczus(ii,jj,is,js,isc,jsc,feld,              &
                        aafin,abfin,bafin,bbfin,                &
                        aafin_n,abfin_n,bafin_n,bbfin_n)

      dimension feld(ii,jj),    asem(is,jj),   bsem(is,jj)
      dimension suma(is,jj),   sumb(is,jj),   sum(ii,jj)
      dimension aafin(is,js), bafin(is,js), abfin(is,js), bbfin(is,js)
      dimension asem_n(is,jj), bsem_n(is,jj)
      dimension suma_n(is,jj), sumb_n(is,jj), sum_n(ii,jj)

      dimension aafin_n(is,js), abfin_n(is,js)
      dimension bafin_n(is,js), bbfin_n(is,js)

      twopi = 8.*atan(1.)
!$omp parallel do &
!$omp private(m,n)
!chacha
      do 10 n = 1,jj
      do 10 m = 1,ii
         sum(m,n)   = 0.
         sum_n(m,n) = 0.
   10 continue
!$omp end parallel do
!chacha
!$omp parallel do &
!$omp private(ms,n)
!chacha
      do 20 n  = 1,jj
      do 20 ms = 1,is
         suma  (ms,n) = 0.
         sumb  (ms,n) = 0.
         asem  (ms,n) = 0.
         bsem  (ms,n) = 0.
         suma_n(ms,n) = 0.
         sumb_n(ms,n) = 0.
         asem_n(ms,n) = 0.
         bsem_n(ms,n) = 0.
   20 continue
!$omp end parallel do

!.....Spectral recomposition: fist in j dimension

!chacha
!$omp parallel do &
!$omp private(ms,n)
!chacha
      do 30 n  = 1,jj
      do 30 ms = 1,isc
      do 25 ns = 2,jsc
       suma(ms,n)   = suma(ms,n)   + aafin(ms,ns)*cos(twopi*real(ns-1)  &
                                  * real(n)/real(jj))                   &
                                  + bafin(ms,ns)*sin(twopi*real(ns-1)   &
                                  * real(n)/real(jj))
       sumb(ms,n)   = sumb(ms,n)   + abfin(ms,ns)*cos(twopi*real(ns-1)  &
                                  * real(n)/real(jj))                   &
                                  + bbfin(ms,ns)*sin(twopi*real(ns-1)   &
                                  * real(n)/real(jj))
       suma_n(ms,n) = suma_n(ms,n) + aafin_n(ms,ns)*cos(twopi*real(ns-1) &
                                 * real(n)/real(jj))                    &
                                  + bafin_n(ms,ns)*sin(twopi*real(ns-1)  &
                                  * real(n)/real(jj))
       sumb_n(ms,n) = sumb_n(ms,n) + abfin_n(ms,ns)*cos(twopi*real(ns-1) &
                                  * real(n)/real(jj))                    &
                                  + bbfin_n(ms,ns)*sin(twopi*real(ns-1)  &
                                  * real(n)/real(jj))
   25 continue
      asem  (ms,n) = aafin  (ms,1) + 2.*suma  (ms,n)
      bsem  (ms,n) = abfin  (ms,1) + 2.*sumb  (ms,n)
      asem_n(ms,n) = aafin_n(ms,1) + 2.*suma_n(ms,n)
      bsem_n(ms,n) = abfin_n(ms,1) + 2.*sumb_n(ms,n)
   30 continue
!$omp end parallel do

!.....Spectral recomposition: fist in i dimension

!chacha
!$omp parallel do &
!$omp private(m,n)
!chacha
      do 40 n  = 1,jj
      do 40 m  = 1,ii
      do 35 ms = 2,isc
         sum(m,n) = sum(m,n)     + asem(ms,n)*cos(twopi*real(ms-1)    &
                                * real(m)/real(ii))                   &
                                + bsem(ms,n)*sin(twopi*real(ms-1)     &
                                * real(m)/real(ii))
         sum_n(m,n) = sum_n(m,n) + asem_n(ms,n)*cos(twopi*real(ms-1)  &
                                * real(m)/real(ii))                   &
                                + bsem_n(ms,n)*sin(twopi*real(ms-1)   &
                                * real(m)/real(ii))
   35 continue
      feld(m,n) = feld(m,n) - asem  (1,n) - 2.*sum  (m,n)   &
                           + asem_n(1,n) + 2.*sum_n(m,n)
   40 continue
!$omp end parallel do
      return
      end subroutine speczus

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!chacha
  subroutine read_wrf_nc( file, u_dum, v_dum)

  implicit none

  include 'netcdf.inc'

  character (len=80) :: file
  character (len=3)              :: go_change
  character (len=10) :: plot_var
  integer :: i,j,k,ivtype, length,length_input
  integer :: plot_dim(3), get_x, get_y, get_z

  character (len=80) :: varnam, att_name, value_chr, print_time
  character (len=80) :: att_sav(10)
  integer :: dimids(10) 
   
  integer cdfid, rcode, id_var, id_att,attlen
  integer nDims, nVars, nAtts, unlimDimID, dims(4),unit_place,order_place
  integer type_to_get, dims3, id_time, n_times, itimes, iatt

  real,    allocatable, dimension(:,:,:) :: data_r
  real,    dimension(:,:,:) :: u_dum, v_dum
  character (len=80) :: times(100)
  integer istart(4), iend(4), middel(4)
  integer istart_t(2), iend_t(2)
  real    sample_value_r, minvalue_r, maxvalue_r
  real, allocatable, dimension(:) :: value_real
  integer sample_value_i, minvalue_i, maxvalue_i, value_int

  length_input=41

  rcode = nf_open(file(1:length_input), NF_NOWRITE, cdfid )
  length = max(1,index(file,' ')-1) 
  if( rcode == 0) then
!    write(6,*) ' '
  else
!    write(6,*) ' error opening netcdf file ',file(1:length)
    stop
  end if

! Get the times first:

  rcode = nf_inq_varid ( cdfid, 'Times', id_var )
    if (rcode .ne. 0) then
!      print*,"This looks like a static file" 
!      print*,"We only print out sample value for this file at the moment"
!      print*,"Also only real and integer fields are printed out"
!      print*,""
      go to 10
    else
      id_time = ncvid( cdfid, 'Times', rcode )
    endif

  rcode = nf_inq_var( cdfid, id_time, varnam, ivtype, ndims, dimids, natts )
  do i=1,ndims 
    rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
  enddo

  n_times = dims(2)
  do i=1,dims(2) 
    istart_t(1) = 1 
    iend_t(1) = dims(1)
    istart_t(2) = i
    iend_t(2) = 1
    rcode = NF_GET_VARA_TEXT  ( cdfid, id_time,  &
                                istart_t, iend_t,    &
                                times(i)          )
  enddo
   
 10 continue          ! come here direct for some static files

  rcode = nf_inq(cdfid, nDims, nVars, nAtts, unlimDimID)

! Get some header information
    do iatt = 1,nAtts
      rcode = nf_inq_attname(cdfid,nf_global,iatt,att_name)
      rcode = nf_inq_att( cdfid,nf_global,att_name,ivtype,attlen )
      if (ivtype == 2) then
        rcode = NF_GET_ATT_TEXT(cdfid, nf_global, att_name, value_chr )
!        write(6,'(A," : ",A)') att_name(1:40),value_chr(1:attlen)
      elseif (ivtype == 4) then
        rcode = NF_GET_ATT_INT(cdfid, nf_global, att_name, value_int )
!        write(6,'(A," : ",i5)') att_name(1:40),value_int
      elseif (ivtype == 5) then
        allocate (value_real(attlen))
        rcode = NF_GET_ATT_REAL(cdfid, nf_global, att_name, value_real )
        if (attlen .gt. 1) then
!          print*,att_name(1:40),": ",value_real
        else
!          write(6,'(A," : ",f12.4)') att_name(1:40),value_real(1)
        endif
        deallocate (value_real)
      endif
    enddo

        do itimes = 1,n_times
!      print*,"  "
!      print_time = times(itimes)
!      print*,"TIME: ",print_time(1:30)
      do id_var = 1,nVars

        dims = 1
        rcode = nf_inq_var( cdfid, id_var, varnam, ivtype, nDims, dimids, nAtts )
        type_to_get = ivtype

        do i=1,ndims
          rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
        enddo

        do id_att = 1,nAtts
          rcode = nf_inq_attname( cdfid,id_var,id_att,att_name )
          rcode = nf_inq_att( cdfid,id_var,att_name,ivtype,attlen )
           if ( ivtype == 4 ) then
             rcode = NF_GET_ATT_INT(cdfid, id_var, att_name, value_int )
           endif
          if ( ivtype == 2 ) then
            rcode = NF_GET_ATT_TEXT(cdfid, id_var, att_name, value_chr )
            att_sav(id_att) = value_chr(1:attlen)
            if (att_name(1:4) == "unit") unit_place = id_att
            if (att_name(1:11) == "MemoryOrder") order_place = id_att
          endif
        enddo

        middel        = 1
        istart        = 1
        istart(nDims) = itimes
        iend          = 1
        do i = 1,nDims-1
          iend(i)     = dims(i)
          middel(i)   = iend(i)/2
        enddo

        if (middel(1) == 0) middel(1) = 1

          allocate (data_r(iend(1),iend(2),iend(3)))
          if( trim(varnam) == 'U' ) then
            call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
            u_dum(:,:,:) = data_r(:,:,:)
          elseif( trim(varnam) == 'V' ) then
            call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
            v_dum(:,:,:) = data_r(:,:,:)
          endif
          deallocate (data_r)

      enddo
    enddo

  call ncclos(cdfid,rcode)

!  print*,"  "
!  print*,"   --- End of input file ---   "

  end subroutine read_wrf_nc

  subroutine read_wrf_lowbdy( file, sst_dum, vegfra_dum, albedo_dum)

  implicit none
  include 'netcdf.inc'

  character (len=80) :: file
  character (len=3)              :: go_change
  character (len=10) :: plot_var 
  integer :: i,j,k,ivtype, length,length_input
  integer :: plot_dim(3), get_x, get_y, get_z

  character (len=80) :: varnam, att_name, value_chr, print_time
  character (len=80) :: att_sav(10) 
  integer :: dimids(10)

  integer cdfid, rcode, id_var, id_att,attlen
  integer nDims, nVars, nAtts, unlimDimID, dims(4),unit_place,order_place
  integer type_to_get, dims3, id_time, n_times, itimes, iatt
    
  real,    allocatable, dimension(:,:,:) :: data_r
  real,    dimension(:,:) :: sst_dum, vegfra_dum, albedo_dum
  character (len=80) :: times(100)
  integer istart(4), iend(4), middel(4)
  integer istart_t(2), iend_t(2)
  real    sample_value_r, minvalue_r, maxvalue_r
  real, allocatable, dimension(:) :: value_real
  integer sample_value_i, minvalue_i, maxvalue_i, value_int

  length_input=41

  rcode = nf_open(file(1:length_input), NF_NOWRITE, cdfid )
  length = max(1,index(file,' ')-1)
  if( rcode == 0) then
!    write(6,*) ' '
  else
!    write(6,*) ' error opening netcdf file ',file(1:length)
    stop
  end if

! Get the times first:

  rcode = nf_inq_varid ( cdfid, 'Times', id_var )
    if (rcode .ne. 0) then
!      print*,"This looks like a static file"
!      print*,"We only print out sample value for this file at the moment"
!      print*,"Also only real and integer fields are printed out"
!      print*,""
      go to 10
    else
      id_time = ncvid( cdfid, 'Times', rcode )
    endif

  rcode = nf_inq_var( cdfid, id_time, varnam, ivtype, ndims, dimids, natts )
  do i=1,ndims
    rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
  enddo

  n_times = dims(2)
  do i=1,dims(2)
    istart_t(1) = 1
    iend_t(1) = dims(1) 
    istart_t(2) = i
    iend_t(2) = 1 
    rcode = NF_GET_VARA_TEXT  ( cdfid, id_time,  &
                                istart_t, iend_t,    &
                                times(i)          )
  enddo

 10 continue          ! come here direct for some static files
    
  rcode = nf_inq(cdfid, nDims, nVars, nAtts, unlimDimID)
  
! Get some header information
    do iatt = 1,nAtts
      rcode = nf_inq_attname(cdfid,nf_global,iatt,att_name)
      rcode = nf_inq_att( cdfid,nf_global,att_name,ivtype,attlen )
      if (ivtype == 2) then
        rcode = NF_GET_ATT_TEXT(cdfid, nf_global, att_name, value_chr )
!        write(6,'(A," : ",A)') att_name(1:40),value_chr(1:attlen)
      elseif (ivtype == 4) then
        rcode = NF_GET_ATT_INT(cdfid, nf_global, att_name, value_int )
!        write(6,'(A," : ",i5)') att_name(1:40),value_int
      elseif (ivtype == 5) then
        allocate (value_real(attlen))
        rcode = NF_GET_ATT_REAL(cdfid, nf_global, att_name, value_real )
        if (attlen .gt. 1) then
!          print*,att_name(1:40),": ",value_real
        else
!          write(6,'(A," : ",f12.4)') att_name(1:40),value_real(1)
        endif
        deallocate (value_real)
      endif
    enddo

        do itimes = 1,n_times
!      print*,"  "
!      print_time = times(itimes)
!      print*,"TIME: ",print_time(1:30)
      do id_var = 1,nVars

        dims = 1
        rcode = nf_inq_var( cdfid, id_var, varnam, ivtype, nDims, dimids, nAtts )
        type_to_get = ivtype

        do i=1,ndims
          rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
        enddo

        do id_att = 1,nAtts
          rcode = nf_inq_attname( cdfid,id_var,id_att,att_name )
          rcode = nf_inq_att( cdfid,id_var,att_name,ivtype,attlen )
           if ( ivtype == 4 ) then
             rcode = NF_GET_ATT_INT(cdfid, id_var, att_name, value_int )
           endif
          if ( ivtype == 2 ) then
            rcode = NF_GET_ATT_TEXT(cdfid, id_var, att_name, value_chr )
            att_sav(id_att) = value_chr(1:attlen)
            if (att_name(1:4) == "unit") unit_place = id_att
            if (att_name(1:11) == "MemoryOrder") order_place = id_att
          endif
        enddo

        middel        = 1
        istart        = 1
        istart(nDims) = itimes
        iend          = 1
        do i = 1,nDims-1
          iend(i)     = dims(i)
          middel(i)   = iend(i)/2
        enddo
        if (middel(1) == 0) middel(1) = 1

          allocate (data_r(iend(1),iend(2),iend(3)))
          if( trim(varnam) == 'TSK' ) then
            call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
            sst_dum(:,:) = data_r(:,:,1)
          elseif( trim(varnam) == 'VEGFRA' ) then
            call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
            vegfra_dum(:,:) = data_r(:,:,1)
          elseif( trim(varnam) == 'ALBBCK' ) then
            call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
            albedo_dum(:,:) = data_r(:,:,1)
          endif
          deallocate (data_r)

      enddo
    enddo

  call ncclos(cdfid,rcode)

  end subroutine read_wrf_lowbdy

END MODULE module_integrate

